@@ -10,11 +10,11 @@ import weakref
 from collections.abc import Mapping
 
 import psycopg2
-from psycopg2 import extras
-from psycopg2.extensions import POLL_ERROR, POLL_OK, POLL_READ, POLL_WRITE
+import psycopg2.extensions
+import psycopg2.extras
 
 from .cursor import Cursor
-from .utils import _ContextManager, create_future, get_running_loop
+from .utils import _ContextManager, create_completed_future, get_running_loop
 
 __all__ = ('connect',)
 
@@ -71,10 +71,11 @@ class Connection:
         self._enable_json = enable_json
         self._enable_hstore = enable_hstore
         self._enable_uuid = enable_uuid
-        self._loop = get_running_loop(kwargs.pop('loop', None) is not None)
-        self._waiter = create_future(self._loop)
+        self._loop = get_running_loop()
+        self._waiter = self._loop.create_future()
 
         kwargs['async_'] = kwargs.pop('async', True)
+        kwargs.pop('loop', None)  # backward compatibility
         self._conn = psycopg2.connect(dsn, **kwargs)
 
         self._dsn = self._conn.dsn
@@ -83,10 +84,7 @@ class Connection:
         self._timeout = timeout
         self._last_usage = self._loop.time()
         self._writing = False
-        self._cancelling = False
-        self._cancellation_waiter = None
         self._echo = echo
-        self._cursor_instance = None
         self._notifies = asyncio.Queue()
         self._weakref = weakref.ref(self)
         self._loop.add_reader(self._fileno, self._ready, self._weakref)
@@ -138,28 +136,27 @@ class Connection:
                 if waiter is not None and not waiter.done():
                     waiter.set_exception(
                         psycopg2.OperationalError("Connection closed"))
-            if state == POLL_OK:
+            if state == psycopg2.extensions.POLL_OK:
                 if self._writing:
                     self._loop.remove_writer(self._fileno)
                     self._writing = False
                 if waiter is not None and not waiter.done():
                     waiter.set_result(None)
-            elif state == POLL_READ:
+            elif state == psycopg2.extensions.POLL_READ:
                 if self._writing:
                     self._loop.remove_writer(self._fileno)
                     self._writing = False
-            elif state == POLL_WRITE:
+            elif state == psycopg2.extensions.POLL_WRITE:
                 if not self._writing:
                     self._loop.add_writer(self._fileno, self._ready, weak_self)
                     self._writing = True
-            elif state == POLL_ERROR:
+            elif state == psycopg2.extensions.POLL_ERROR:
                 self._fatal_error("Fatal error on aiopg connection: "
                                   "POLL_ERROR from underlying .poll() call")
             else:
-                self._fatal_error("Fatal error on aiopg connection: "
-                                  "unknown answer {} from underlying "
-                                  ".poll() call"
-                                  .format(state))
+                self._fatal_error(f"Fatal error on aiopg connection: "
+                                  f"unknown answer {state} from underlying "
+                                  f".poll() call")
 
     def _fatal_error(self, message):
         # Should be called from exception handler only.
@@ -173,39 +170,19 @@ class Connection:
 
     def _create_waiter(self, func_name):
         if self._waiter is not None:
-            if self._cancelling:
-                if not self._waiter.done():
-                    raise RuntimeError('%s() called while connection is '
-                                       'being cancelled' % func_name)
-            else:
-                raise RuntimeError('%s() called while another coroutine is '
-                                   'already waiting for incoming '
-                                   'data' % func_name)
-        self._waiter = create_future(self._loop)
+            raise RuntimeError(f'{func_name}() called while another coroutine '
+                               f'is already waiting for incoming data')
+        self._waiter = self._loop.create_future()
         return self._waiter
 
     async def _poll(self, waiter, timeout):
         assert waiter is self._waiter, (waiter, self._waiter)
         self._ready(self._weakref)
 
-        async def cancel():
-            self._waiter = create_future(self._loop)
-            self._cancelling = True
-            self._cancellation_waiter = self._waiter
-            self._conn.cancel()
-            if not self._conn.isexecuting():
-                return
-            try:
-                await asyncio.wait_for(self._waiter, timeout)
-            except psycopg2.extensions.QueryCanceledError:
-                pass
-            except asyncio.TimeoutError:
-                self._close()
-
         try:
             await asyncio.wait_for(self._waiter, timeout)
         except (asyncio.CancelledError, asyncio.TimeoutError) as exc:
-            await asyncio.shield(cancel())
+            await asyncio.shield(self.close())
             raise exc
         except psycopg2.extensions.QueryCanceledError as exc:
             self._loop.call_exception_handler({
@@ -215,19 +192,14 @@ class Connection:
             })
             raise asyncio.CancelledError
         finally:
-            if self._cancelling:
-                self._cancelling = False
-                if self._waiter is self._cancellation_waiter:
-                    self._waiter = None
-                self._cancellation_waiter = None
-            else:
-                self._waiter = None
+            self._waiter = None
 
     def _isexecuting(self):
         return self._conn.isexecuting()
 
     def cursor(self, name=None, cursor_factory=None,
-               scrollable=None, withhold=False, timeout=None):
+               scrollable=None, withhold=False, timeout=None,
+               isolation_level=None):
         """A coroutine that returns a new cursor object using the connection.
 
         *cursor_factory* argument can be used to create non-standard
@@ -237,24 +209,17 @@ class Connection:
         *name*, *scrollable* and *withhold* parameters are not supported by
         psycopg in asynchronous mode.
 
-        NOTE: as of [TODO] any previously created created cursor from this
-            connection will be closed
         """
+
         self._last_usage = self._loop.time()
         coro = self._cursor(name=name, cursor_factory=cursor_factory,
                             scrollable=scrollable, withhold=withhold,
-                            timeout=timeout)
+                            timeout=timeout, isolation_level=isolation_level)
         return _ContextManager(coro)
 
     async def _cursor(self, name=None, cursor_factory=None,
-                      scrollable=None, withhold=False, timeout=None):
-
-        if not self.closed_cursor:
-            warnings.warn(('You can only have one cursor per connection. '
-                           'The cursor for connection will be closed forcibly'
-                           ' {!r}.').format(self), ResourceWarning)
-
-        self.free_cursor()
+                      scrollable=None, withhold=False, timeout=None,
+                      isolation_level=None):
 
         if timeout is None:
             timeout = self._timeout
@@ -263,8 +228,10 @@ class Connection:
                                        cursor_factory=cursor_factory,
                                        scrollable=scrollable,
                                        withhold=withhold)
-        self._cursor_instance = Cursor(self, impl, timeout, self._echo)
-        return self._cursor_instance
+        cursor = Cursor(
+            self, impl, timeout, self._echo, isolation_level
+        )
+        return cursor
 
     async def _cursor_impl(self, name=None, cursor_factory=None,
                            scrollable=None, withhold=False):
@@ -287,29 +254,14 @@ class Connection:
                 self._loop.remove_writer(self._fileno)
 
         self._conn.close()
-        self.free_cursor()
 
         if self._waiter is not None and not self._waiter.done():
             self._waiter.set_exception(
                 psycopg2.OperationalError("Connection closed"))
 
-    @property
-    def closed_cursor(self):
-        if not self._cursor_instance:
-            return True
-
-        return bool(self._cursor_instance.closed)
-
-    def free_cursor(self):
-        if not self.closed_cursor:
-            self._cursor_instance.close()
-            self._cursor_instance = None
-
     def close(self):
         self._close()
-        ret = create_future(self._loop)
-        ret.set_result(None)
-        return ret
+        return create_completed_future(self._loop)
 
     @property
     def closed(self):
@@ -360,18 +312,8 @@ class Connection:
             "tpc_recover cannot be used in asynchronous mode")
 
     async def cancel(self):
-        """Cancel the current database operation."""
-        if self._waiter is None:
-            return
-
-        async def cancel():
-            self._conn.cancel()
-            try:
-                await self._waiter
-            except psycopg2.extensions.QueryCanceledError:
-                pass
-
-        await asyncio.shield(cancel())
+        raise psycopg2.ProgrammingError(
+            "cancel cannot be used in asynchronous mode")
 
     async def reset(self):
         raise psycopg2.ProgrammingError(
@@ -484,22 +426,13 @@ class Connection:
         return self._echo
 
     def __repr__(self):
-        msg = (
-            '<'
-            '{module_name}::{class_name} '
-            'isexecuting={isexecuting}, '
-            'closed={closed}, '
-            'echo={echo}, '
-            'cursor={cursor}'
-            '>'
-        )
-        return msg.format(
-            module_name=type(self).__module__,
-            class_name=type(self).__name__,
-            echo=self.echo,
-            isexecuting=self._isexecuting(),
-            closed=bool(self.closed),
-            cursor=repr(self._cursor_instance)
+        return (
+            f'<'
+            f'{type(self).__module__}::{type(self).__name__} '
+            f'isexecuting={self._isexecuting()}, '
+            f'closed={self.closed}, '
+            f'echo={self.echo}, '
+            f'>'
         )
 
     def __del__(self):
@@ -509,7 +442,7 @@ class Connection:
             return
         if _conn is not None and not _conn.closed:
             self.close()
-            warnings.warn("Unclosed connection {!r}".format(self),
+            warnings.warn(f"Unclosed connection {self!r}",
                           ResourceWarning)
 
             context = {'connection': self,
@@ -548,18 +481,18 @@ class Connection:
     async def _connect(self):
         try:
             await self._poll(self._waiter, self._timeout)
-        except Exception:
-            self.close()
+        except BaseException:
+            await asyncio.shield(self.close())
             raise
         if self._enable_json:
-            extras.register_default_json(self._conn)
+            psycopg2.extras.register_default_json(self._conn)
         if self._enable_uuid:
-            extras.register_uuid(conn_or_curs=self._conn)
+            psycopg2.extras.register_uuid(conn_or_curs=self._conn)
         if self._enable_hstore:
             oids = await self._get_oids()
             if oids is not None:
                 oid, array_oid = oids
-                extras.register_hstore(
+                psycopg2.extras.register_hstore(
                     self._conn,
                     oid=oid,
                     array_oid=array_oid
@@ -574,4 +507,4 @@ class Connection:
         return self
 
     async def __aexit__(self, exc_type, exc_val, exc_tb):
-        self.close()
+        await self.close()
